<?php

use Acms\Services\Facades\Storage;
use Acms\Services\Update\System\CheckForUpdate;
use Acms\Services\Facades\Process;

class ACMS_POST_Update_Exec extends ACMS_POST
{
    public function post()
    {
        if (!sessionWithAdministration()) die();
        if ('update' <> ADMIN) die();
        if (RBID !== BID) die();
        if (SBID !== BID) die();

        ignore_user_abort(true);
        set_time_limit(0);

        header("HTTP/1.1 301");
        header('location: ' . HTTP_REQUEST_URL);
        $this->run();
        die();
    }

    /**
     * Run update.
     */
    protected function run()
    {
        $manager = Process::newProcessManager();
        $manager->addTask(function () {
            set_time_limit(0);

            $logger = App::make('update.logger');
            $downloadService = App::make('update.download');
            $placeFileService = App::make('update.place.file');
            $checkUpdateService = App::make('update.check');
            $dest = ARCHIVES_DIR . uniqueString() . '/';
            $success = false;

            DB::setThrowException(true);

            try {
                $logger->init();
                $range = CheckForUpdate::PATCH_VERSION;
                if ( config('system_update_range') === 'minor' ) {
                    $range = CheckForUpdate::MINOR_VERSION;
                }
                if ( $checkUpdateService->checkUseCache(phpversion(), $range) ) {
                    $version = $checkUpdateService;
                } else {
                    throw new \RuntimeException(gettext('パッケージ情報の取得に失敗しました'));
                }

                $url = $version->getPackageUrl();
                $root_dir = $version->getRootDir();

                $new_path = $dest . $root_dir;
                $backup_dir = 'private/' . 'backup' . date('YmdHis') . '/';
                if (!Storage::isWritable('private')) {
                    throw new \RuntimeException(gettext('privateディレクトリに書き込み権限を与えてください'));
                }

                // validate
                $placeFileService->validate($new_path, $backup_dir);

                // System Update
                $downloadService->download($dest, $url);
                $placeFileService->exec($new_path, $backup_dir);

                $success = true;
            } catch ( \Exception $e ) {
                $message = $e->getMessage();
                if ( !empty($message) ) {
                    $logger->error($e->getMessage());
                }
                sleep(3);
                $logger->terminate();
            }

            DB::setThrowException(false);
            $logger->addMessage(gettext('ダウンロードファイルを削除中...'), 0);
            Storage::removeDirectory($dest);

            if ( $success ) {
                $logger->fileUpdateSuccess();
            }
        });
        $manager->run();
    }
}